﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Web
Imports System.Web.Mvc
Imports MsdnMvcWebGrid.Domain
Imports MsdnMvcWebGrid.Infrastructure
Imports MsdnMvcWebGrid.Models.Product

Namespace MsdnMvcWebGrid.Controllers
    Public Class ProductController
        Inherits Controller
        Private ReadOnly _productService As IProductService
    ' NOTE: In the magazine article I showed a "List" action, but to easily demonstrate the various approaches I have changed the action name to match the approach being shown (e.g. BasicWebGrid, ...)
    ' NOTE: would typically add a DI container and Dependency resolver to remove the need for this constructor
    Public Sub New()
            Me.New(New Domain.EfProductService())

    End Sub
    Public Sub New(ByVal productService As IProductService)
        _productService = productService
    End Sub

    Public Function Details(ByVal id As Integer) As ActionResult
        Dim model As ProductDetailsModel = New ProductDetailsModel With {.Product = _productService.GetProduct(id, includeCategory:=True)}
        Return View(model)
    End Function

    Public Function BasicWebGrid() As ActionResult
        Dim model As IEnumerable(Of Product) = _productService.GetProducts()

        Return View(model)
    End Function
    Public Function BasicWebGridWebForms() As ActionResult
        Dim model As IEnumerable(Of Product) = _productService.GetProducts()

        Return View(model)
    End Function


    Public Function BasicStronglyTypedWebGrid() As ActionResult
        Dim model As IEnumerable(Of Product) = _productService.GetProducts()

        Return View(model)
    End Function

    Public Function BasicStronglyTypedWebGridWebForms() As ActionResult
        Dim model As IEnumerable(Of Product) = _productService.GetProducts()

        Return View(model)
    End Function


    Public Function DefaultPagingAndSorting() As ActionResult
        Dim model As IEnumerable(Of Product) = _productService.GetProducts()

        Return View(model)
    End Function
    Public Function ServerPaging(Optional ByVal page As Integer = 1) As ActionResult
        Const pageSize As Integer = 5

        Dim totalRecords As Integer
        Dim products As IEnumerable(Of Product) = _productService.GetProducts(totalRecords, pageSize:=pageSize, pageIndex:=page - 1)

        Dim model As PagedProductsModel = New PagedProductsModel With {.PageSize = pageSize, .PageNumber = page, .Products = products, .TotalRows = totalRecords}
        Return View(model)
    End Function
    Public Function ServerPagingAndSorting(Optional ByVal page As Integer = 1, Optional ByVal sort As String = "Name", Optional ByVal sortDir As String = "Ascending") As ActionResult
        Const pageSize As Integer = 5

        Dim totalRecords As Integer
        Dim products As IEnumerable(Of Product) = _productService.GetProducts(totalRecords, pageSize:=pageSize, pageIndex:=page - 1, sort:=sort, sortOrder:=GetSortDirection(sortDir))

        Dim model As PagedProductsModel = New PagedProductsModel With {.PageSize = pageSize, .PageNumber = page, .Products = products, .TotalRows = totalRecords}
        Return View(model)
    End Function

    Public Function StandardAjax(Optional ByVal page As Integer = 1, Optional ByVal sort As String = "Name", Optional ByVal sortDir As String = "Ascending") As ActionResult
        Const pageSize As Integer = 5

        Dim totalRecords As Integer
        Dim products As IEnumerable(Of Product) = _productService.GetProducts(totalRecords, pageSize:=pageSize, pageIndex:=page - 1, sort:=sort, sortOrder:=GetSortDirection(sortDir))

        Dim model As PagedProductsModel = New PagedProductsModel With {.PageSize = pageSize, .PageNumber = page, .Products = products, .TotalRows = totalRecords}
        Return View(model)
    End Function

    Public Function CustomAjax(Optional ByVal page As Integer = 1, Optional ByVal sort As String = "Name", Optional ByVal sortDir As String = "Ascending") As ActionResult
        Const pageSize As Integer = 5

        Dim totalRecords As Integer
        Dim products As IEnumerable(Of Product) = _productService.GetProducts(totalRecords, pageSize:=pageSize, pageIndex:=page - 1, sort:=sort, sortOrder:=GetSortDirection(sortDir))

        Dim model As PagedProductsModel = New PagedProductsModel With {.PageSize = pageSize, .PageNumber = page, .Products = products, .TotalRows = totalRecords}
        If Request.IsAjaxRequest() Then
            Return PartialView("CustomAjax_Grid", model)
        End If
        Return View(model)
    End Function


        Private Function GetSortDirection(ByVal sortDirection As String) As SortDirection
            If sortDirection <> Nothing Then
                If sortDirection.Equals("DESC", StringComparison.OrdinalIgnoreCase) OrElse sortDirection.Equals("DESCENDING", StringComparison.OrdinalIgnoreCase) Then
                    'sortDirection.Descending()
                    Return Domain.SortDirection.Ascending

                End If
            End If
            'sortDirection.Ascending()
            Return Domain.SortDirection.Descending

        End Function
    End Class
End Namespace