﻿Imports System
Imports System.Collections.Generic
Imports System.Data.Objects
Imports System.Linq
Imports System.Linq.Expressions
Imports System.Web

Namespace MsdnMvcWebGrid.Domain
    Public Class EfProductService
        Implements IProductService
        ' helpers that take an IQueryable<Product> and a bool to indicate ascending/descending
        ' and apply that ordering to the IQueryable and return the result
        Private ReadOnly _productOrderings As IDictionary(Of String, Func(Of IQueryable(Of Product), Boolean, IOrderedQueryable(Of Product))) = New Dictionary(Of String, Func(Of IQueryable(Of Product), Boolean, IOrderedQueryable(Of Product))) From {{"Name", CreateOrderingFunc(Of Product, String)(Function(p) p.Name)}, {"ListPrice", CreateOrderingFunc(Of Product, Decimal?)(Function(p) p.ListPrice)}}
        ''' <summary>
        ''' returns a Func that takes an IQueryable and a bool, and sorts the IQueryable (ascending or descending based on the bool).
        ''' The sort is performed on the property identified by the key selector.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <typeparam name="TKey"></typeparam>
        ''' <param name="keySelector"></param>
        ''' <returns></returns>
        Private Shared Function CreateOrderingFunc(Of T, TKey)(ByVal keySelector As Expression(Of Func(Of T, TKey))) As Func(Of IQueryable(Of T), Boolean, IOrderedQueryable(Of T))
            Return Function(source, ascending) If(ascending, source.OrderBy(keySelector), source.OrderByDescending(keySelector))
        End Function


        Public Function GetProduct(ByVal productId As Integer, Optional ByVal includeCategory As Boolean = False) As Product Implements IProductService.GetProduct

            Using context As New AdventureWorksLTEntities()
                Dim products As ObjectQuery(Of Product) = context.Products
                If includeCategory Then
                    products = products.Include("Category")
                End If
                Return products.FirstOrDefault(Function(p) p.ProductId = productId)
            End Using
        End Function
        Public Function GetProducts() As IEnumerable(Of Product) Implements IProductService.GetProducts
            Using context As New AdventureWorksLTEntities()
                Return context.Products.OrderBy(Function(p) p.Name).ToList()
            End Using
        End Function

        Public Function GetProducts(<System.Runtime.InteropServices.Out()> ByRef totalRecords As Integer, Optional ByVal pageSize As Integer = -1, Optional ByVal pageIndex As Integer = -1, Optional ByVal sort As String = "Name", Optional ByVal sortOrder As SortDirection = SortDirection.Ascending) As IEnumerable(Of Product) Implements IProductService.GetProducts
            Using context As New AdventureWorksLTEntities()
                Dim products As IQueryable(Of Product) = context.Products

                totalRecords = products.Count()

                ' apply sorting
                Dim applyOrdering As Func(Of IQueryable(Of Product), Boolean, IOrderedQueryable(Of Product)) = _productOrderings(sort)
                products = applyOrdering(products, sortOrder = SortDirection.Ascending)

                If pageSize > 0 AndAlso pageIndex >= 0 Then
                    ' apply paging
                    products = products.Skip(pageIndex * pageSize).Take(pageSize)
                End If

                Return products.ToList()
            End Using
        End Function
    End Class
End Namespace