﻿Imports System.Collections.Generic
Imports System.Web.Mvc

Namespace MsdnMvcWebGrid.Infrastructure
    Public Module WebGridExtensions
        ''' <summary>
        ''' Light-weight wrapper around the constructor for WebGrid so that we get take advantage of compiler type inference
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="htmlHelper"></param>
        ''' <param name="source">Data source</param>
        ''' <param name="columnNames">Data source column names. Auto-populated by default.</param>
        ''' <param name="defaultSort">Default sort column.</param>
        ''' <param name="rowsPerPage">Number of rows per page.</param>
        ''' <param name="canPage">true to enable paging</param>
        ''' <param name="canSort">true to enable sorting</param>
        ''' <param name="ajaxUpdateContainerId">ID for the grid's container element. This enables AJAX support.</param>
        ''' <param name="ajaxUpdateCallback">Callback function for the AJAX functionality once the update is complete</param>
        ''' <param name="fieldNamePrefix">Prefix for query string fields to support multiple grids.</param>
        ''' <param name="pageFieldName">Query string field name for page number.</param>
        ''' <param name="selectionFieldName">Query string field name for selected row number.</param>
        ''' <param name="sortFieldName">Query string field name for sort column.</param>
        ''' <param name="sortDirectionFieldName">Query string field name for sort direction.</param>
        ''' <returns></returns>
        <System.Runtime.CompilerServices.Extension()> _
        Public Function Grid(Of T)(ByVal htmlHelper As HtmlHelper, ByVal source As IEnumerable(Of T), Optional ByVal columnNames As IEnumerable(Of String) = Nothing, Optional ByVal defaultSort As String = Nothing, Optional ByVal rowsPerPage As Integer = 10, Optional ByVal canPage As Boolean = True, Optional ByVal canSort As Boolean = True, Optional ByVal ajaxUpdateContainerId As String = Nothing, Optional ByVal ajaxUpdateCallback As String = Nothing, Optional ByVal fieldNamePrefix As String = Nothing, Optional ByVal pageFieldName As String = Nothing, Optional ByVal selectionFieldName As String = Nothing, Optional ByVal sortFieldName As String = Nothing, Optional ByVal sortDirectionFieldName As String = Nothing) As WebGrid(Of T)
            Return New WebGrid(Of T)(source, columnNames, defaultSort, rowsPerPage, canPage, canSort, ajaxUpdateContainerId, ajaxUpdateCallback, fieldNamePrefix, pageFieldName, selectionFieldName, sortFieldName, sortDirectionFieldName)
        End Function

        ''' <summary>
        ''' Light-weight wrapper around the constructor for WebGrid so that we get take advantage of compiler type inference and to automatically call Bind to disable the automatic paging and sorting (use this for server-side paging)
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="htmlHelper"></param>
        ''' <param name="source"></param>
        ''' <param name="totalRows"></param>
        ''' <param name="columnNames"></param>
        ''' <param name="defaultSort"></param>
        ''' <param name="rowsPerPage"></param>
        ''' <param name="canPage"></param>
        ''' <param name="canSort"></param>
        ''' <param name="ajaxUpdateContainerId"></param>
        ''' <param name="ajaxUpdateCallback"></param>
        ''' <param name="fieldNamePrefix"></param>
        ''' <param name="pageFieldName"></param>
        ''' <param name="selectionFieldName"></param>
        ''' <param name="sortFieldName"></param>
        ''' <param name="sortDirectionFieldName"></param>
        ''' <returns></returns>
        <System.Runtime.CompilerServices.Extension()> _
        Public Function ServerPagedGrid(Of T)(ByVal htmlHelper As HtmlHelper, ByVal source As IEnumerable(Of T), _
                                              ByVal totalRows As Integer, Optional ByVal columnNames As IEnumerable(Of String) = Nothing, Optional ByVal defaultSort As String = Nothing, Optional ByVal rowsPerPage As Integer = 10, _
                                              Optional ByVal canPage As Boolean = True, Optional ByVal canSort As Boolean = True, Optional ByVal ajaxUpdateContainerId As String = Nothing, Optional _
                                              ByVal ajaxUpdateCallback As String = Nothing, Optional _
                                              ByVal fieldNamePrefix As String = Nothing, Optional ByVal pageFieldName As String = Nothing, _
                                              Optional ByVal selectionFieldName As String = Nothing, Optional ByVal sortFieldName _
                                              As String = Nothing, Optional ByVal sortDirectionFieldName As String = Nothing) As WebGrid(Of T)
            Dim webGrid = New WebGrid(Of T) _
                          (Nothing, columnNames, defaultSort, rowsPerPage, canPage, canSort, ajaxUpdateContainerId, ajaxUpdateCallback, _
                           fieldNamePrefix, pageFieldName, selectionFieldName, sortFieldName, sortDirectionFieldName)
            Return webGrid.Bind(source, rowCount:=totalRows, autoSortAndPage:=False)

        End Function
    End Module
End Namespace